open Js_of_ocaml
open Common

let error s =
  Format.eprintf "%s@." s ;
  assert false

let quickstart canvas_id w h =
  let canvas_opt =
    Dom_html.window##.document##getElementById (Js.string canvas_id)
  in
  let canvas =
    Js.coerce_opt canvas_opt Dom_html.CoerceTo.canvas (fun _ ->
        error "can't find canvas")
  in
  canvas##.width := w ;
  canvas##.height := h ;
  Obj.magic (canvas##getContext Dom_html._2d_)

module M : Common.Backend.S = struct
  type t = Dom_html.canvasRenderingContext2D Js.t

  type audio_file = Dom_html.audioElement Js.t

  let begin_display ctx = ctx##beginPath

  let display ctx = ctx##stroke

  let clear ctx =
    let w = ctx##.canvas##.clientWidth in
    let h = ctx##.canvas##.clientHeight in
    let w, h = Point.to_float (w, h) in
    ctx##fillRect 0. 0. w h

  let draw_point ctx p =
    let x, y = Point.to_float p in
    ctx##fillRect x y 1. 1.

  let draw_line ctx p1 p2 =
    let x1, y1 = Point.to_float p1 in
    let x2, y2 = Point.to_float p2 in
    ctx##moveTo x1 y1 ;
    ctx##lineTo x2 y2

  let draw_rect ctx p v =
    let x, y = Point.to_float p in
    let w, h = Point.to_float v in
    ctx##rect x y w h

  let error = error

  let fill_rect ctx p v =
    let x, y = Point.to_float p in
    let w, h = Point.to_float v in
    ctx##fillRect x y w h

  let init_audio () = ()

  let load_audio_file path =
    let audio = Dom_html.createAudio Dom_html.window##.document in
    audio##.src := Js.string path ;
    audio

  let play_audio_once audio = audio##play

  let play_audio_loop audio =
    audio##.loop := Js.bool true ;
    audio##play

  let js_color_of_rgb (r, g, b) =
    Js.string (Format.sprintf "rgb(%d, %d, %d)" r g b)

  let js_color_of_rgba (r, g, b, a) =
    Js.string (Format.sprintf "rgba(%d, %d, %d, %d)" r g b a)

  let set_rgb_color ctx color =
    let color = js_color_of_rgb color in
    ctx##.strokeStyle := color ;
    ctx##.fillStyle := color

  let set_rgba_color ctx color =
    let color = js_color_of_rgba color in
    ctx##.strokeStyle := color ;
    ctx##.fillStyle := color
end
